/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "debug.h"
#include "ni-koala.h"
#include "nil-c64.h"


static int
_unpack(const uint8_t *src, const uint8_t *src_end, uint8_t *dst, const uint8_t *dst_end)
{
    int count = 0;
    for(;;){
        int b;
        if(count<=0){
            if(src >= src_end)
                return -1;
            b = *src++;
            if(0xfe==b){
                if(src+2 > src_end)
                    return -1;
                b = *src++;
                count = *src++;
            }else{
                count = 1;
            }
        }
        *dst++ = b;
        if(dst >= dst_end){
            DEBUG_p("bytes left: %d",(int)(dst_end - dst));
            return 0;
        }
        --count;
    }
}


int
ni_koala_decode(nile_t *ni,const uint8_t *data,size_t len)
{
    const nil_c64_vram_t *pvram;
    DEBUG_p("begin (size = %d)",(int)len);
    if(data[0] != 0x00 || data[1] != 0x60){
        DEBUG_p("incorrect header");
        return -1;
    }
    nile_ensure_indexed(ni, 160, 200);
    if(10003 == len){
        /* uncompressed */
        pvram = (nil_c64_vram_t*)&data[2];
    }else{
        /* compressed */
        nil_c64_vram_t vram;
        if(_unpack((uint8_t*)data+2, (uint8_t*)data+len, (uint8_t*)&vram, (uint8_t*)(&vram + 1))<0){
            DEBUG_p("uncompress error");
            return -1;
        }
        pvram = &vram;
    }
    return nil_c64_vram_decode(ni, pvram);
}
