/*
 * Copyright (C) 2000 – 2020  LibreOffice contributors
 * Copyright (C) 2021         Magnus Bergman <magnus@snisurset.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#include "chart.h"


CGMChart *cgm_chart_init(CGMChart *chart)
{
    int i;
    chart->mnCurrentFileType = 0;
    loa_array_init(&chart->text_entry_list,sizeof(TextEntry));
    for(i=0;i<7;++i){
        data_node_init(&chart->mDataNode[i]);
        chart->mDataNode[i].nZoneEnum = i;
    }
    return chart;
}


void cgm_chart_done(CGMChart *chart)
{
    int i;
    for(i=0;i<loa_array_length(&chart->text_entry_list);++i)
        text_entry_done((TextEntry*)loa_array_get(&chart->text_entry_list, i));
    loa_array_done(&chart->text_entry_list);
}


void CGMChart_InsertTextEntry(CGMChart *chart, TextEntry *pTextEntry )
{
    TextEntry *dst = (TextEntry*)loa_array_append(&chart->text_entry_list, 1);
    memcpy(dst, pTextEntry, sizeof(TextEntry));
}


void CGMChart_ResetAnnotation(CGMChart *chart)
{
    chart->mDataNode[0].nZoneEnum = 0;
}


bool CGMChart_IsAnnotation(CGMChart *chart)
{
    return (chart->mDataNode[0].nZoneEnum == 0 );
}
