/*
 * Copyright (C) 2019 - 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
extern "C"{
#define new new_
#include <abydos-plugin.h>
#undef new
#include <debug.h>
#include <string.h>
}

//#include <ImfInputFile.h>
#include <ImfArray.h>
#include <ImfChannelList.h>
#include <ImfFrameBuffer.h>
#include <ImfMultiPartInputFile.h>
#include <ImfInputPart.h>
#include <ImfRgbaFile.h>
#include <deque>

using namespace Imf;
using namespace Imath;
using namespace std;

#if 0
struct layer_t{
    char *name;
    half *pixels;
    layer_t(const char *s,int len,int width,int height){
        name = strndup(s,len);
        pixels = new half[4 * width * height];
    }
    layer_t(layer_t &&other)
        :name(move(other.name)),pixels(move(other.pixels))
    {
        other.name = NULL;
        other.pixels = NULL;
    }
    ~layer_t(){
        if(name)
            free(name);
        if(pixels)
            delete[] pixels;
    }
};
#endif

struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
    //deque<layer_t> layers;
    /*
    layer_t *get_layer(const char *name, int len)
    {
        for(auto &layer : layers){
            DEBUG_p("name: %s",layer.name);
            if(!strncmp(layer.name,name,len))
                return &layer;
        }
        layers.emplace_front(name,len,info->width,info->height);
        return &layers.front();
    }
    */
    void create_surface(Array<Rgba> &pixels){
        surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,info->width,info->height);
        uint32_t *dst = (uint32_t*)cairo_image_surface_get_data(surface);
        int rowskip = cairo_image_surface_get_stride(surface)/sizeof(uint32_t)-info->width;
        int offs = 0;
        for(int y=0;y<info->height;++y){
            for(int x=0;x<info->width;++x){
                double a = pixels[offs].a;
                *dst++ = ((int)(255*a*pixels[offs].r))<<16|
                         ((int)(255*a*pixels[offs].g))<< 8|
                         ((int)(255*a*pixels[offs].b))<< 0|
                         ((int)(255*a))<<24;
                ++offs;
            }
            dst += rowskip;
        }
        cairo_surface_mark_dirty(surface);
    }
    int load(const char* filename){
        try{
            MultiPartInputFile file(filename);
            InputPart part(file,0);
            const ChannelList channels = part.header().channels();
            if (channels.findChannel("Y")){
                info->error = "Cannot handle YCA images";
                return -1;
            }

            const Box2i dw = part.header().dataWindow();
            info->width = dw.max.x - dw.min.x + 1;
            info->height = dw.max.y - dw.min.y + 1;
            size_t size = info->width * info->height;

            Array<Rgba> pixels;
            pixels.resizeErase (size);
            memset (pixels, 0, size * (sizeof(Rgba)));

#if 0
            for (ChannelList::ConstIterator i = channels.begin(); i != channels.end(); ++i){
                const char *name = i.name();
                DEBUG_p("channel: %s",name);
                const char *pos = strchr(name,'.');
                int len;
                if(pos){
                    len = pos-name;
                    ++pos;
                }else{
                    len = 0;
                    pos = name;
                }
                layer_t *layer = get_layer(name,len);
                DEBUG_p("layer: '%s'",layer->name);
            }
#endif

            FrameBuffer frameBuffer;
            frameBuffer.insert("R", Slice(HALF, (char*)&pixels[0].r, sizeof(Rgba), info->width * sizeof(Rgba), 1, 1, 0.0));
            frameBuffer.insert("G", Slice(HALF, (char*)&pixels[0].g, sizeof(Rgba), info->width * sizeof(Rgba), 1, 1, 0.0));
            frameBuffer.insert("B", Slice(HALF, (char*)&pixels[0].b, sizeof(Rgba), info->width * sizeof(Rgba), 1, 1, 0.0));
            frameBuffer.insert("A", Slice(HALF, (char*)&pixels[0].a, sizeof(Rgba), info->width * sizeof(Rgba), 1, 1, 1.0));
            part.setFrameBuffer(frameBuffer);
            part.readPixels(dw.min.y, dw.max.y);

            create_surface(pixels);

            return 0;
        }catch(const std::exception &e){
            info->error = "Failed to load image file";
            return -1;
        }
    }
#if 0
    void blend(){
        if(!surface)
            surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,info->width,info->height);
        uint32_t *dst = (uint32_t*)cairo_image_surface_get_data(surface);
        int rowskip = cairo_image_surface_get_stride(surface)/sizeof(uint32_t)-info->width;
        int offs = 0;
        for(int y=0;y<info->height;++y){
            for(int x=0;x<info->width;++x){
                double r = 0;
                double g = 0;
                double b = 0;
                double a = 0;
                for(auto &layer : layers){
                    double a1 = layer.pixels[offs+0];
                    double a0 = 1-a1;
                    r = a0*r + a1*layer.pixels[offs+1];
                    g = a0*g + a1*layer.pixels[offs+2];
                    b = a0*b + a1*layer.pixels[offs+3];
                    a = a0*a + a1;
                }
                offs += 4;
                *dst++ = ((int)(255*a*r))<<16|
                         ((int)(255*a*g))<< 8|
                         ((int)(255*a*b))<< 0|
                         ((int)(255*a))<<24;
            }
            dst += rowskip;
        }
        cairo_surface_mark_dirty(surface);
    }
#endif
};


extern "C" const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = [](){
            static const char *mime_types[] = {
                "image/x-exr",
                NULL
            };
            return mime_types;
        },
        .new_ = [](const char *mime_type, abydos_plugin_info_t *info){
            auto *h = new abydos_plugin_handle_t;
            h->info = info;
            h->info->threadsafe = 1;
            h->surface = NULL;
            return h;
        },
        .free = [](abydos_plugin_handle_t *h){
            if(h->surface)
                cairo_surface_destroy(h->surface);
            delete h;
        },
        .create_from_file = [](abydos_plugin_handle_t *h,const char *filename){
            return h->load(filename);
        },
        .get_image_surface = [](abydos_plugin_handle_t *h,
                                int page,
                                cairo_bool_t *layer_visibility,
                                int variant,
                                double time,
                                int frame)
        {
            /* FIXME!! check if layer_visibility is different */
            //if(!h->surface)
            //    h->blend();
            return cairo_surface_reference(h->surface);
        }
    };
    return &p;
}
