/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "lottiecairo.h"


static int
_render_node(const LOTNode *n, cairo_t *cr)
{
    const float *p = n->mPath.ptPtr;
    const float *end = p+n->mPath.ptCount;
    int i;

    /* FIXME!! return if no stroke or fill is to be performed */

    for(i=0;i<n->mPath.elmCount;++i){
        switch(n->mPath.elmPtr[i]){
        case 0:
            if(p+2 > end)
                return -1;
            cairo_move_to(cr,p[0],p[1]);
            p += 2;
            break;
        case 1:
            if(p+2 > end)
                return -1;
            cairo_line_to(cr,p[0],p[1]);
            p += 2;
            break;
        case 2:
            if(p+6 > end)
                return -1;
            cairo_curve_to(cr,p[0],p[1],p[2],p[3],p[4],p[5]);
            p += 6;
            break;
        case 3:
            cairo_close_path(cr);
            break;
        }
    }

    cairo_set_source_rgba(cr,(double)n->mColor.r/255,
                             (double)n->mColor.g/255,
                             (double)n->mColor.b/255,
                             (double)n->mColor.a/255);

    if(n->mStroke.enable){
        cairo_set_line_width(cr,n->mStroke.width);
        switch(n->mStroke.cap){
        case CapFlat:
            cairo_set_line_cap(cr,CAIRO_LINE_CAP_BUTT);
            break;
        case CapSquare:
            cairo_set_line_cap(cr,CAIRO_LINE_CAP_SQUARE);
            break;
        case CapRound:
            cairo_set_line_cap(cr,CAIRO_LINE_CAP_ROUND);
            break;
        }
        switch(n->mStroke.join){
        case JoinMiter:
            cairo_set_line_join(cr,CAIRO_LINE_JOIN_MITER);
            cairo_set_miter_limit(cr,n->mStroke.miterLimit);
            break;
        case JoinBevel:
            cairo_set_line_join(cr,CAIRO_LINE_JOIN_BEVEL);
            break;
        case JoinRound:
            cairo_set_line_join(cr,CAIRO_LINE_JOIN_ROUND);
            break;
        }
        /* FIXME!! handle dash */
        cairo_stroke(cr);
    }else{
        /* FIXME!! is this the correct way to handle fill? */
        cairo_fill(cr);
    }
    return 0;
}


int
lottie_cairo_render(const LOTLayerNode *n, cairo_t *cr)
{
    int i;
    for(i=0;i<n->mLayerList.size;++i)
        lottie_cairo_render(n->mLayerList.ptr[i],cr);

    for(i=0;i<n->mNodeList.size;++i)
        _render_node(n->mNodeList.ptr[i],cr);

    return 0;
}
