/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <debug.h>
#include <string.h>                     /* memset() */

#include "t42.h"


static int
_hamming_8_4_decode(int b)
{
    static const int hamming_8_4_inverse[] = {
        0x01,   -1, 0x01, 0x01,   -1, 0x00, 0x01,   -1,
          -1, 0x02, 0x01,   -1, 0x0a,   -1,   -1, 0x07,
          -1, 0x00, 0x01,   -1, 0x00, 0x00,   -1, 0x00,
        0x06,   -1,   -1, 0x0b,   -1, 0x00, 0x03,   -1,
          -1, 0x0c, 0x01,   -1, 0x04,   -1,   -1, 0x07,
        0x06,   -1,   -1, 0x07,   -1, 0x07, 0x07, 0x07,
        0x06,   -1,   -1, 0x05,   -1, 0x00, 0x0d,   -1,
        0x06, 0x06, 0x06,   -1, 0x06,   -1,   -1, 0x07,
          -1, 0x02, 0x01,   -1, 0x04,   -1,   -1, 0x09,
        0x02, 0x02,   -1, 0x02,   -1, 0x02, 0x03,   -1,
        0x08,   -1,   -1, 0x05,   -1, 0x00, 0x03,   -1,
          -1, 0x02, 0x03,   -1, 0x03,   -1, 0x03, 0x03,
        0x04,   -1,   -1, 0x05, 0x04, 0x04, 0x04,   -1,
          -1, 0x02, 0x0f,   -1, 0x04,   -1,   -1, 0x07,
          -1, 0x05, 0x05, 0x05, 0x04,   -1,   -1, 0x05,
        0x06,   -1,   -1, 0x05,   -1, 0x0e, 0x03,   -1,
          -1, 0x0c, 0x01,   -1, 0x0a,   -1,   -1, 0x09,
        0x0a,   -1,   -1, 0x0b, 0x0a, 0x0a, 0x0a,   -1,
        0x08,   -1,   -1, 0x0b,   -1, 0x00, 0x0d,   -1,
          -1, 0x0b, 0x0b, 0x0b, 0x0a,   -1,   -1, 0x0b,
        0x0c, 0x0c,   -1, 0x0c,   -1, 0x0c, 0x0d,   -1,
          -1, 0x0c, 0x0f,   -1, 0x0a,   -1,   -1, 0x07,
          -1, 0x0c, 0x0d,   -1, 0x0d,   -1, 0x0d, 0x0d,
        0x06,   -1,   -1, 0x0b,   -1, 0x0e, 0x0d,   -1,
        0x08,   -1,   -1, 0x09,   -1, 0x09, 0x09, 0x09,
          -1, 0x02, 0x0f,   -1, 0x0a,   -1,   -1, 0x09,
        0x08, 0x08, 0x08,   -1, 0x08,   -1,   -1, 0x09,
        0x08,   -1,   -1, 0x0b,   -1, 0x0e, 0x03,   -1,
          -1, 0x0c, 0x0f,   -1, 0x04,   -1,   -1, 0x09,
        0x0f,   -1, 0x0f, 0x0f,   -1, 0x0e, 0x0f,   -1,
        0x08,   -1,   -1, 0x05,   -1, 0x0e, 0x0d,   -1,
          -1, 0x0e, 0x0f,   -1, 0x0e, 0x0e,   -1, 0x0e
    };
    b = hamming_8_4_inverse[b];
    if(b < 0)
        DEBUG_p("data error");
    return b;
}


#define get8(a) (_hamming_8_4_decode((a)[0])|_hamming_8_4_decode((a)[1])<<4)
#define get16(a) (get8(a)|get8(a+2)<<8)

int
t42_parse(const uint8_t *data, int len, t42_packet_func_t userfunc, void *userdata)
{
    int page = 0;
    //int subcode = 0;
    while(len >= 42){
        uint8_t packet[40];
        int header = get8(data);
        //int magazine = header & 7;
        int row =  header >> 3;
        int ret;
        int i;
        DEBUG_p("row: %d",row);
        switch(row){
        case 0:
            page = get8(data+2);
            //subcode = get16(data+4) & 0x3f7f;
            memset(packet, ' ', 8);
            i = 8;
            break;
        case 26:
            /* FIXME!! */
            i = -1;
            break;
        case 27:
            /* FIXME!! */
            i = -1;
            break;
        case 28:
            /* FIXME!! */
            i = -1;
            break;
        default:
            if(row < 26){
                i = 0;
            }else{
                /* error? */
                i = -1;
            }
            break;
        }
        if(i >= 0){
            for(;i<40;++i)
                packet[i] = data[i+2] & 0x7f;
            ret = userfunc(userdata, page, row, packet);
            if(ret)
                return ret;
        }
        data += 42;
        len -= 42;
    }
    return 0;
}
