/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <ctype.h>
#include <string.h>

#include "tti7.h"

static const char*
_strechr(const char *s, const char *end, int c)
{
    for(;s<end && *s!=c;++s);
    return s;
}


static const char*
_strechrs(const char *s, const char *end, const char *cs)
{
    for(;s<end && !strchr(cs,*s);++s);
    return s;
}


int tti7_parse(const char *s, int len, tti7_line_func_t userfunc, void *userdata){
    const char *end = s+len;
    for(;;){
        const char *lineend = _strechrs(s, end, "\r\n");
        const char *p;
        if(lineend >= end)
            return 0;
        p = _strechr(s, lineend, ',');
        if(2==p-s){
            tti7_line_t line;
            if(!memcmp(s, "CT", 2)){
                line = TTI7_CT;
            }else if(!memcmp(s, "DE", 2)){
                line = TTI7_DE;
            }else if(!memcmp(s, "OL", 2)){
                line = TTI7_OL;
            }else if(!memcmp(s, "PN", 2)){
                line = TTI7_PN;
            }else if(!memcmp(s, "PS", 2)){
                line = TTI7_PS;
            }else{
                line = TTI7_NONE;
            }
            if(line){
                str_t str[16];
                int num = 0;
                for(;;){
                    if(s >= lineend){
                        int ret = userfunc(userdata, line, num, str);
                        if(ret)
                            return ret;
                        break;
                    }
                    s = p+1;
                    p = _strechr(s, lineend, ',');
                    str[num].str = s;
                    str[num].len = p-s;
                    ++num;
                }
            }
        }
        s = lineend;
        while(s < end && isspace(*s))
            ++s;
    }
}
