/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
extern "C"{
#define new new_
#include <abydos-plugin.h>
#include <debug.h>
#undef new
}
#include <libwpg/WPGraphics.h>

#include "revenge-cairo.h"


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static int
_wpg_create_from_stream(abydos_plugin_handle_t *h, RVNGInputStream *stream)
{
    h->surface = cairo_recording_surface_create(CAIRO_CONTENT_COLOR_ALPHA, NULL);
    RVNGCairoDrawingGenerator gen(h->surface, &h->info->width, &h->info->height);
    bool res = libwpg::WPGraphics::parse(stream, &gen);
    return res ? 0 : -1;
}


static void
_wpg_render(abydos_plugin_handle_t *h,
            cairo_t *cr,
            int page,
            cairo_bool_t *layer_visibility,
            int variant,
            double time,
            int frame)
{
    cairo_set_source_surface (cr, h->surface, 0, 0);
    cairo_paint (cr);
}


extern "C" const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = [](){
            static const char *mime_types[] = {
                "image/x-wpg",
                NULL
            };
            return mime_types;
        },
        .new_ = [](const char *mime_type, abydos_plugin_info_t *info){
            auto *h = new abydos_plugin_handle_t;
            info->threadsafe = 1;
            h->info = info;
            h->surface = NULL;
            return h;
        },
        .free = [](abydos_plugin_handle_t *h)
        {
            if(h->surface)
                cairo_surface_destroy(h->surface);
            delete h;
        },
        .create_from_data = [](abydos_plugin_handle_t *h,const char *data, size_t len)
        {
            RVNGStringStream stream((const unsigned char*)data, len);
            return _wpg_create_from_stream(h, &stream);
        },
        .create_from_file = [](abydos_plugin_handle_t *h,const char *filename)
        {
            RVNGFileStream stream(filename);
            return _wpg_create_from_stream(h, &stream);
        },
        .render = _wpg_render
    };
    return &p;
}
