/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef transform_h
#define transform_h

#include <cairo.h>
#include <loa-array.h>

typedef struct{
    loa_array_t matrix_stack;
}transform_t;

#define transform_init(t) loa_array_init(&(t)->matrix_stack, sizeof(cairo_matrix_t))
#define transform_done(t) loa_array_done(&(t)->matrix_stack)
#define transform_size(t) loa_array_length(&(t)->matrix_stack)
cairo_matrix_t *transform_push_identity(transform_t *t);
cairo_matrix_t *transform_push_translate(transform_t *t, double x, double y);
cairo_matrix_t *transform_push_multiply(transform_t *t, cairo_matrix_t *m);
void transform_pop(transform_t *t);
void transform_translate(transform_t *t, double x, double y);
void transform_multiply(transform_t *t, cairo_matrix_t *m);
void transform_apply_pos(transform_t *t, double *rx, double *ry);
void transform_apply_size(transform_t *t, double *rw, double *rh);

#endif /* transform_h */
