/*
 * Copyright (C) 2017  Teemu Suutari
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 */
#include "dec_fbr2.h"
#include "streami.h"
#include "streamo.h"


void FBR2Decompressor::decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify)
{
	ForwardInputStream inputStream(_packedData,0,_packedData.size());

	ForwardOutputStream outputStream(rawData,0,rawData.size());

	uint8_t mode=inputStream.readByte();
	while (!outputStream.eof())
	{
		bool doCopy=false;
		uint32_t count=0;
		switch (mode)
		{
			case 33:
			count=uint32_t(inputStream.readByte())<<24;
			count|=uint32_t(inputStream.readByte())<<16;
			count|=uint32_t(inputStream.readByte())<<8;
			count|=uint32_t(inputStream.readByte());
			if (count>=0x8000'0000)
			{
				doCopy=true;
				count=-count;
			}
			break;

			case 67:
			count=uint32_t(inputStream.readByte())<<8;
			count|=uint32_t(inputStream.readByte());
			if (count>=0x8000)
			{
				doCopy=true;
				count=0x10000-count;
			}
			break;

			case 100:
			count=uint32_t(inputStream.readByte());
			if (count>=0x80)
			{
				doCopy=true;
				count=0x100-count;
			}
			break;

			default:
			throw Decompressor::DecompressionError();
		}

		count++;
		if (doCopy) {
			for (uint32_t i=0;i<count;i++) outputStream.writeByte(inputStream.readByte());
		} else {
			uint8_t repeatChar=inputStream.readByte();
			for (uint32_t i=0;i<count;i++) outputStream.writeByte(repeatChar);
		}
	}
}
