/*
 * Copyright (C) 2020 - 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include <string.h>                     /* memcpy() memmove() */

#include "nil-cycle.h"


int
nil_cycle_total_frames(nil_cycle_t *cc, int count)
{
    if(NIL_CYCLE_OFF == cc->direction){
        return 1;
    }else{
        return cc->last - cc->first + 1;
    }
}


double
nil_cycle_total_duration(nil_cycle_t *cc, int count)
{
    if(NIL_CYCLE_OFF == cc->direction){
        return 0;
    }else{
        return cc->duration * (cc->last - cc->first + 1);
    }
}


void
nil_cycle_apply_step(nil_cycle_t *cc, void *palette, int colorsize)
{
    char color[16];
    switch(cc->direction){
    case NIL_CYCLE_OFF:
        break;
    case NIL_CYCLE_LEFT:
        memcpy(color, palette + cc->first*colorsize, colorsize);
        memmove(palette + cc->first*colorsize, palette + (cc->first+1)*colorsize, (cc->last - cc->first)*colorsize);
        memcpy(palette + cc->last*colorsize, color, colorsize);
        break;
    case NIL_CYCLE_RIGHT:
        memcpy(color, palette + cc->last*colorsize, colorsize);
        memmove(palette + (cc->first+1)*colorsize, palette + cc->first*colorsize, (cc->last - cc->first)*colorsize);
        memcpy(palette + cc->first*colorsize, color, colorsize);
        break;
    }
}


void
nil_cycle_apply_time(nil_cycle_t *cc, double time, void *palette, int colorsize)
{
    while(time >= cc->duration){
        nil_cycle_apply_step(cc, palette, colorsize);
        time -= cc->duration;
    }
}
