/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef iff_h
#define iff_h

#include <endian.h>
#include <stdint.h>
#include <stdio.h>

#if __BYTE_ORDER == __BIG_ENDIAN
# define IFF_ID(a,b,c,d) ((a<<24)|(b<<16)|(c<<8)|(d<<0))
#else
# define IFF_ID(a,b,c,d) ((a<<0)|(b<<8)|(c<<16)|(d<<24))
#endif

#define IFF_FORM IFF_ID('F','O','R','M')

/* FORM types */
#define IFF_AMFF IFF_ID('A','M','F','F')
#define IFF_ANIM IFF_ID('A','N','I','M')
#define IFF_DPST IFF_ID('D','P','S','T')
#define IFF_ILBM IFF_ID('I','L','B','M')
#define IFF_PBM  IFF_ID('P','B','M',' ')
#define IFF_VDLT IFF_ID('V','D','L','T')

typedef int(*nil_iff_parse_func_t)(void *userdata, uint32_t id, const char*, size_t);

typedef struct{
    FILE *f;
    off_t lenpos;
}ni_iff_chunk_t;

int nil_iff_parse(uint32_t id, int adj_form_len, const char*, size_t len, nil_iff_parse_func_t, void *userdata);

void nil_iff_chunk_begin(ni_iff_chunk_t*, FILE*, uint32_t id);
void nil_iff_chunk_end(ni_iff_chunk_t*);

#endif /* iff_h */
